#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np

from room.miner import MinerModule, Miner
from room.modules.miner.estimator.nnw import NeuralNetWork as NNW
from room.utils.config import network_config

class NNWModule(MinerModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder6']['back']),
            send_addr=int(network_config['forwarder5']['front']),
            recv_title='nnw',
            send_title='nnw',
            miner=NeuralNetWork(),
        )


class NeuralNetWork(Miner):

    def __init__(self):
        self.appliance = 'viera'        
        self.nnw = NNW(self.appliance)

    def mining(self, data):
        predict = self.predict(data['data'])
        return self.recommend(data['target'][self.appliance], predict)

    def predict(self, data):
        data = np.array([data]).astype(np.float32)        
        return self.nnw.predict(data)[0]

    def recommend(self, current, predict):
        if predict is not None and current != predict:            
            return [{'appliance': self.appliance, 'method': predict}]
        else:
            return []


if __name__ == "__main__":
    process = NNWModule()
    process.run()
    
    




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from room import parser
from room.utils.config import network_config

class DefaultParserModule(parser.ParserModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder1']['back']),
            send_addr=int(network_config['forwarder2']['front']),                        
            recv_title='default',
            send_title='',
            parser=DefaultParser()
        )

class DefaultParser(parser.Parser):

    def parse(self, data):
        return data

    
if __name__ == "__main__":
    process = DefaultParserModule()
    process.run()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from room import parser
from room.utils.config import network_config

class ApplianceParserModule(parser.ParserModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder1']['back']),
            send_addr=int(network_config['forwarder2']['front']),
            recv_title='appliance_status',
            send_title='event',
            category='appliance',
            parser=ApplianceParser()
        )

class ApplianceParser(parser.Parser):
        
    def parse(self, data):
        data.pop('time')
        return [{key: data[key]} for key in data.keys()]
    
if __name__ == "__main__":
    process = ApplianceParserModule()
    process.run()
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import unittest
import json

from room.mining.naive_cf_data import NaiveCFData

@unittest.skip("TODO")
class NaiveCFDataTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.naive_cf_data = NaiveCFData()

    def setUp(self):
        data = json.loads('{"appliances": {"CeilingLight1": 0.0, "CeilingLight2": 0.0, "CeilingLight3": 0.0, "airCleaner": 0.0, "airConditioner": 0.0, "airConditionerFront": 0.0, "aircleaner001": 0.0, "aircon001": 0.0, "allLight": 0.0, "allLight001": 0.0, "aroma": 0.0, "aroma001": 0.0, "dvd001": 0.0, "fan": 0.0, "fan001": 0.0, "floorLight": 0.0, "floorLight001": 0.0, "frontCeilingLight": 0.0, "hddJukeBox": 0.0, "hddRecorder": 0.0, "hotCarpet001": 0.0, "newAirCleaner": 0.0, "speaker002": 0.0, "tableLight": 0.0, "tableLight001": 0.0, "timestamp": 1349068860000, "tv": 0.0, "tv001": 0.0, "viera": 0.0, "windAirCon001": 0.0}, "sensors": {"airflow": -0.1375886637, "brightness": -0.1612054489, "humidity": 0.1882105153, "out_brightness": -0.2450407919, "out_humidity": 0.1086926153, "out_pressure": 0.0142628603, "out_temperature": 0.0239133782, "people": -0.1040588553, "temperature": 0.1238222059, "timestamp": 1349068860000, "w_humidity": 0.1882098846, "w_light": -0.1504021764, "w_sound": -0.0020367455, "w_temperature_c": 0.112190812, "w_temperature_f": 0.0762658346}, "timestamp": "2015-11-11 17:33:02.958697+09:00"}')
        self.naive_cf_data.add(data)

    def test_all(self):
        result = self.naive_cf_data.all()

    def test_add(self):
        data = json.loads('{"appliances": {"CeilingLight1": 0.0, "CeilingLight2": 0.0, "CeilingLight3": 0.0, "airCleaner": 0.0, "airConditioner": 0.0, "airConditionerFront": 0.0, "aircleaner001": 0.0, "aircon001": 0.0, "allLight": 0.0, "allLight001": 0.0, "aroma": 0.0, "aroma001": 0.0, "dvd001": 0.0, "fan": 0.0, "fan001": 0.0, "floorLight": 0.0, "floorLight001": 0.0, "frontCeilingLight": 0.0, "hddJukeBox": 0.0, "hddRecorder": 0.0, "hotCarpet001": 0.0, "newAirCleaner": 0.0, "speaker002": 0.0, "tableLight": 0.0, "tableLight001": 0.0, "timestamp": 1349068860000, "tv": 0.0, "tv001": 0.0, "viera": 0.0, "windAirCon001": 0.0}, "sensors": {"airflow": -0.1375886637, "brightness": -0.1612054489, "humidity": 0.1882105153, "out_brightness": -0.2450407919, "out_humidity": 0.1086926153, "out_pressure": 0.0142628603, "out_temperature": 0.0239133782, "people": -0.1040588553, "temperature": 0.1238222059, "timestamp": 1349068860000, "w_humidity": 0.1882098846, "w_light": -0.1504021764, "w_sound": -0.0020367455, "w_temperature_c": 0.112190812, "w_temperature_f": 0.0762658346}, "timestamp": "2015-11-11 17:33:02.958697+09:00"}')
        self.naive_cf_data.add(data)
        
    def test_sensors(self):
        result = self.naive_cf_data.sensors()
        
    def test_appliances(self):
        result = self.naive_cf_data.appliances()
        
    def test_set(self):
        result = self.naive_cf_data.set()

    def tearDown(self):
        self.naive_cf_data.drop()
        
    @classmethod
    def tearDownClass(cls):
        cls.naive_cf_data.drop()        
        
if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

from room.buffer import BufferModule, StateHandler
from room.state import State
from room.utils.config import config
from room.utils.config import network_config

class DataBufferModule(BufferModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder2']['back']),
            send_addr=int(network_config['forwarder3']['front']),
            recv_title='',
            send_title='buffer',
            state_handler=DataStateHandler(),
            period=int(config['data_buffer']['interval_ms'])
        )
        
class DataStateHandler(StateHandler):

    def __init__(self):
        self._state = State()
        
    def dump(self):
        return self._state.dump_at_now()

    def buffering(self, data):
        category = data['category']
        key,value = list(data['msg'].items())[0]

        if category == 'sensor':
            self._state.update_sensor(key, value)
        elif category == 'inout':
            self._state.update_inout(key, value)           
        elif category == 'appliance':
            self._state.update_appliance(key, value)



if __name__ == "__main__":
    process = DataBufferModule()
    process.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from room.miner import MinerModule, Miner
from room.modules.miner.estimator.speed import SPEED
from room.utils.config import network_config

class SpeedModule(MinerModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder3']['back']),
            send_addr=int(network_config['forwarder4']['front']),
            recv_title='speed',
            send_title='speed',
            miner=SpeedPack(),
        )


class SpeedPack(Miner):

    def __init__(self):
        self.speed = SPEED()

    def mining(self, data):
        result = self.speed.execute(list(data.items())[0])
        if result:
            return {result[0]:result[1]} # tuple to dict
        else:
            return []


if __name__ == "__main__":
    process = SpeedModule()
    process.run()
    
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest
import json
import pytz
from datetime import datetime
from room.buffer.state import State

class StateTest(unittest.TestCase):

    def test_get_sensor_keys(self):
        state = State()        
        state.update_sensor('temp', 22)
        state.update_sensor('humid', 23)
        state.update_sensor('pressure', 23)   
        self.assertEqual(len(state.get_sensor_keys()), 3)
        self.assertTrue('temp' in list(state.get_sensor_keys()))
        self.assertTrue('humid' in list(state.get_sensor_keys()))
        self.assertTrue('pressure' in list(state.get_sensor_keys()))
        
    def test_get_appliance_keys(self):
        state = State()        
        state.update_appliance('tv', 1)
        state.update_appliance('tv', 0)
        state.update_appliance('light', 1)                       
        self.assertEqual(len(state.get_appliance_keys()), 2)
        self.assertTrue('tv' in list(state.get_appliance_keys()))
        self.assertTrue('light' in list(state.get_appliance_keys()))
    
    def test_update_sensor(self):
        state = State()
        state.update_sensor('temp', 22)
        state.update_sensor('humid', 23)
        state.update_sensor('pressure', 23)
        self.assertEqual(state.get_sensor_value('temp'), 22)
        self.assertEqual(state.get_sensor_value('humid'), 23)
        self.assertEqual(state.get_sensor_value('pressure'), 23)
        
    def test_update_appliance(self):
        state = State()
        state.update_appliance('tv', 1)
        state.update_appliance('tv', 0)
        state.update_appliance('light', 1)          
        self.assertEqual(len(state.get_appliance_keys()), 2)
        self.assertEqual(state.get_appliance_value('tv'), 0)
        self.assertEqual(state.get_appliance_value('light'), 1)
        

    def test_to_json(self):
        state = State()
        state.update_sensor('temp', 22)
        state.update_sensor('humid', 23)
        state.update_sensor('pressure', 23)
        state.update_appliance('tv', 1)
        state.update_appliance('tv', 0)
        state.update_appliance('light', 1)                  
        result = state.to_json()
        self.assertTrue(isinstance(result, str))
        result = json.loads(result)
        self.assertEqual(result['sensors']['temp'], 22)
        self.assertEqual(result['sensors']['humid'], 23)
        self.assertEqual(result['sensors']['pressure'], 23)
        self.assertEqual(result['appliances']['tv'], 0)
        self.assertEqual(result['appliances']['light'], 1)
        
    def test_to_json_withn_timestamp(self):
        state = State()
        state.update_sensor('temp', 22)
        state.update_sensor('humid', 23)
        state.update_sensor('pressure', 23)
        state.update_appliance('tv', 1)
        state.update_appliance('tv', 0)
        state.update_appliance('light', 1)
        tz = pytz.timezone('Asia/Tokyo')
        now = datetime.now(tz)        
        result = state.to_json_with_timestamp(now)
        self.assertTrue(isinstance(result, str))
        result = json.loads(result)
        self.assertEqual(result['timestamp'], str(now))
        self.assertEqual(result['sensors']['temp'], 22)
        self.assertEqual(result['sensors']['humid'], 23)
        self.assertEqual(result['sensors']['pressure'], 23)
        self.assertEqual(result['appliances']['tv'], 0)
        self.assertEqual(result['appliances']['light'], 1)


    def test_to_json_at_now(self):
        state = State()
        state.update_sensor('temp', 22)
        state.update_sensor('humid', 23)
        state.update_sensor('pressure', 23)
        state.update_appliance('tv', 1)
        state.update_appliance('tv', 0)
        state.update_appliance('light', 1)
        tz = pytz.timezone('Asia/Tokyo')
        now = datetime.now(tz)        
        result = state.to_json_at_now()
        self.assertTrue(isinstance(result, str))
        result = json.loads(result)
        self.assertEqual(result['timestamp'].split(' ')[0], str(now).split(' ')[0]) # 年月日の一致
        self.assertEqual(result['timestamp'].split(' ')[1].split('.')[0], str(now).split(' ')[1].split('.')[0]) #時間の一致
        self.assertEqual(result['sensors']['temp'], 22)
        self.assertEqual(result['sensors']['humid'], 23)
        self.assertEqual(result['sensors']['pressure'], 23)
        self.assertEqual(result['appliances']['tv'], 0)
        self.assertEqual(result['appliances']['light'], 1)

if __name__ == '__main__':
    unittest.main()        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest
import requests

from room.router.router import Application

@unittest.skip("アプリケーションが終了しないため")
class RouterTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._app = Application()
        cls._app.start()
    
    def test_main(self):
        #r = requests.get('localhost:8888')
        #print(r)
        #self.assertEqual(1,1)
        self.assertTrue(True)
        
    def test_datahandler(self):
        self.assertTrue(True)
        
    @classmethod
    def tearDownClass(cls):
        print('test')
        cls._app.stop()
        
if __name__ == '__main__':
    unittest.main()        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest
import json

from room.parser.sparkcore_parser import SparkCoreParserModule, SparkCoreParser
from room.utils.publisher import Publisher
from room.tests.utils.subscriber import Subscriber

@unittest.skip("コルーチン化しないとテストできないっぽい")
class SparkCoreParserModuleTest(unittest.TestCase)            :

    @classmethod
    def setUpClass(cls):
        cls.publisher = Publisher("127.0.0.1:5558")
        cls.sparkcore_parser = SparkCoreParserModule("127.0.0.1:5558")
        cls.sparkcore_parser.run()
        cls.subscriber = Subscriber('127.0.0.1:5557')

    def test_sparkcore_parser_module(self):

        data = """
        ["sparkcore",{
        "sensorId": "core04",
        "temperature": 27.6,
        "humidity": 41.200001,
        "pressure": 991.455566,
        "brightness": 34.848484,
        "pir_count": 0
        }]"""

        self.publisher().send('sparkcore', 'parse', data)
        result = self.subscribe.recv()
        print(result)
        self.assertTrue(True)

    @classmethod
    def tearDownClass(cls):
        print('tearDown!!!') 
        cls.sparkcore_parser.stop()        
        
class SparkCoreParserTest(unittest.TestCase):

    def test_parser(self):
        parser = SparkCoreParser()
        data = """
        ["sparkcore",{
        "sensorId": "core04",
        "temperature": 27.6,
        "humidity": 41.200001,
        "pressure": 991.455566,
        "brightness": 34.848484,
        "pir_count": 0
        }]"""
        results = parser.parse(json.loads(data))
        expections = ['{"core04_temperature": 27.6}',
                      '{"core04_humidity": 41.200001}',
                      '{"core04_pressure": 991.455566}',
                      '{"core04_brightness": 34.848484}',
                      '{"core04_pir_count": 0}']
        for result in results:
            self.assertEqual(result[0], 'sensor')
            self.assertTrue(result[1] in expections)


        
if __name__ == '__main__':
    unittest.main()        




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymongo import MongoClient

from room import database
from room.utils.config import config
from room.utils.config import network_config

class MongoModule(database.DatabaseModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder3']['back']),
            recv_title='cbr',
            db=MongoDB()
        )

class MongoDB(database.Database):

    def __init__(self):
        self._client     = MongoClient(config['mongo']['host'], int(config['mongo']['port']))
        self._db         = self._client[config['mongo']['db']]
        self._collection = self._db[config['mongo']['collection']]
        
    def save(self, data):
        self._collection.insert_one(data)

    def load(self):
        return [record for record in self._collection.find()]

    def drop(self):
        self._collection.delete_many({})

if __name__ == "__main__":
    process = MongoModule()
    process.run()
            



#!/usr/bin/env python3
# -*- coding: utf-8- 

import unittest
import zmq
import time
from zmq.utils import jsonapi as json

from room.utils.publisher import Publisher
from room.tests.utils.subscriber import Subscriber

@unittest.skip("コルーチン化しないとテストできないっぽい")
class PublisherTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.publisher = Publisher('127.0.0.1:5556')
        cls.subscriber = Subscriber('127.0.0.1:5556', b'test')        
        
    def test_send(self):
        self.publisher.send('test', 'method', '{"temp": 20, "humid": 30}')
        result = self.subscriber.recv()
        self.assertEqual(result[0], b'test') # data_type
        
        msg = json.loads(result[1])
        self.assertEqual(msg[0], 'method')
        
        data = json.loads(msg[1])
        self.assertEqual(data['temp'], 20)
        self.assertEqual(data['humid'], 30)        

    @classmethod
    def tearDownClass(cls):
        cls.publisher.stop()
        cls.subscriber.stop()
        
if __name__ == '__main__':
    unittest.main()        
        



# -*- coding: utf-8 -*-

import os
import configparser

config = configparser.ConfigParser()
network_config = configparser.ConfigParser()

config.read(os.environ['PYTHONPATH'] + '/config.ini')
network_config.read(os.environ['PYTHONPATH'] + '/network.ini')



#!/ur/bin/env python3
# -*- coding: utf-8 -*-

import zmq
from abc import ABCMeta, abstractmethod

from room import zmq_base as base

class DatabaseModule(base.ZmqProcess):
    
    def __init__(self, recv_addr, recv_title, db):
        super().__init__()
        self.sub_stream = None        
        self.recv_addr  = recv_addr
        self.recv_title = recv_title
        self.db = db
    
    def setup(self):
        super().setup()
        self.sub_stream, _ = self.stream(zmq.SUB, self.recv_addr, bind=False, subscribe=self.recv_title.encode('utf-8'))
        self.sub_stream.on_recv(SubStreamHandler(self.sub_stream, self.stop, self.db))

    def run(self):
        self.setup()
        print('Start loop!')
        self.loop.start()

    def stop(self):
        self.loop.stop()
        
class SubStreamHandler(base.MessageHandler):

    def __init__(self, sub_stream, stop, db):
        super().__init__()
        self._sub_stream = sub_stream
        self._stop = stop
        self._db = db
        
    def execute(self, data):
        self._db.save(data)
        
    def stop(self, data):
        self._stop()

        
class Database(metaclass=ABCMeta):

    @abstractmethod
    def save(self, data):
        raise NotImplementedError()
    
    @abstractmethod
    def load(self):
        raise NotImplementedError()        
    



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import zmq
import time
from zmq.eventloop.ioloop import PeriodicCallback
from abc import ABCMeta,abstractmethod

from room import zmq_base as base
from room.publisher import Publisher
from room.utils.config import config
from room.utils.log import logging

class BufferModule(base.ZmqProcess):

    def __init__(self, recv_addr, send_addr, recv_title, send_title, state_handler, period=1000):
        super().__init__()
        self.sub_stream    = None
        self.timer         = None        
        self.recv_addr     = recv_addr
        self.recv_title    = recv_title
        self.state_handler = state_handler
        self.period        = period
        self.callback      = Callback(Publisher(send_addr, send_title), self.state_handler)
        
    def setup(self):
        super().setup() 
        self.sub_stream, _ = self.stream(zmq.SUB, self.recv_addr, bind=False, subscribe=self.recv_title.encode('utf-8'))
        self.sub_stream.on_recv(SubStreamHandler(self.sub_stream, self.stop, self.state_handler))
        self.timer = PeriodicCallback(self.callback, self.period, self.loop)

    def run(self):
        self.setup()
        print('Start loop!')
        self.timer.start()        
        self.loop.start()

    def stop(self):
        self.loop.stop()

        
class SubStreamHandler(base.MessageHandler):
    
    def __init__(self, sub_stream, stop, state_handler):
        super().__init__()
        self._sub_stream = sub_stream
        self._stop = stop
        self._state_handler = state_handler

    def execute(self, data):
        self._state_handler.buffering(data)
        
    def stop(self, data):
        self._stop()


class StateHandler(metaclass=ABCMeta):

    @abstractmethod
    def dump(self):
        raise NotImplementedError()

    @abstractmethod    
    def buffering(self, data):
        raise NotImplementedError()        

    
class Callback(object):

    def __init__(self, publisher, state_handler):
        self.publisher = publisher
        self.state_handler = state_handler

    def __call__(self):
        data = self.state_handler.dump()
        self.publisher.send(data)
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import zmq
import json
from zmq.eventloop.ioloop import PeriodicCallback
from abc import ABCMeta, abstractmethod

from room import zmq_base as base
from room.publisher import Publisher
from room.utils.log import logging
from room.utils.config import config


class ParserModule(base.ZmqProcess):

    def __init__(self, recv_addr, send_addr, recv_title, send_title, category, parser):
        '''
        @param category: {'sensor', 'appliance'}

        '''
        super().__init__()
        self.sub_stream = None        
        self.recv_addr = recv_addr
        self.send_addr = send_addr
        self.recv_title = recv_title
        self.send_title = send_title
        self.category = category
        self.parser = parser

    def setup(self):
        super().setup()
        self.sub_stream, _ = self.stream(zmq.SUB, self.recv_addr, bind=False, subscribe=self.recv_title.encode('utf-8'))
        self.sub_stream.on_recv(SubStreamHandler(self.sub_stream, self.stop, self.send_addr, self.send_title, self.category, self.parser))

    def run(self):
        self.setup()
        print('Start loop!')
        self.loop.start()

    def stop(self):
        self.loop.stop()

class SubStreamHandler(base.MessageHandler):
    
    def __init__(self, sub_stream, stop, send_addr, send_title, category, parser):
        super().__init__()
        self._sub_stream = sub_stream
        self._stop = stop
        self._parser = parser
        self._send_title = send_title
        self._category = category
        self._publisher = Publisher(send_addr)

    def execute(self, data):
        parsed_data = self._parser.parse(json.loads(data))
        for msg in parsed_data:
            self._publisher.send({'category': self._category, 'msg':msg}, self._send_title)
        
    def stop(self, data):
        self._stop()


class Parser(metaclass=ABCMeta):

    @abstractmethod
    def parse(self, data):
        '''
        @return [tuple(category, state)]

        state must {"key": "value"}
        '''
        raise NotImplementedError()




#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest
from zmq.utils import jsonapi as json

from room.buffer.default_buffer import DefaultBufferModule, DefaultStateHandler
from room.tests.utils.subscriber import Subscriber

@unittest.skip("TODO")
class DefaultStateHandlerTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.state_handler = DefaultStateHandler()
        cls.subscriber = Subscriber('127.0.0.1:5556')
        
    def test_update_sensor(self):
        data = {'test_temperature': 22}
        self.state_handler.update_sensor(data)
        result = self.state_handler.get_sensor()
        self.assertEqual(result['test_temperature'], 22)
        
    def test_update_appliance(self):
        data = {'viera': 1}
        self.state_handler.update_appliance(data)
        result = self.state_handler.get_appliance()
        self.assertEqual(result['viera'], 1)

    @unittest.skip("コルーチン化しないとテストできないっぽい")        
    def test_call(self):
        self.state_handler.update_sensor({'test_temperature': 22})
        self.state_handler.update_appliance({'viera': 1})        
        self.state_handler.__call__()

        result = self.subscriber.recv()
        
        msg = json.loads(result[1])
        self.assertEqual(msg[0], 'mining')
        
        data = json.loads(msg[1])
        self.assertEqual(data['sensors']['test_temperature'], 22)                
        self.assertEqual(data['appliances']['viera'], 1)

    @classmethod
    def tearDownClass(cls):
        cls.subscriber.stop()

        
if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from datetime import datetime
import pytz

class State:
    '''
    センサや家電の状態を保持するクラス

    '''
    def __init__(self):
        self._sensor_state = {} # sensorの状態を保持する辞書
        self._inout_state = {} # 入退室の状態を保持する辞書
        self._applianece_state = {} # applianceの状態を保持する辞書
        
    def __str__(self):
        return str(self.dump_at_now())

    def format_sensors(self, keys, value):
        for key in keys:
            self._sensor_state[key] = value

    def format_appliances(self, keys,value):
        for key in keys:
            self._applianece_state[key] = value

    def format_inout(self, keys, value):
        for key in keys:
            self._inout_state[key] = value

    def update_sensor(self, key, value):
        '''
        sensorの状態を更新する

        '''
        self._sensor_state[key] = value

    def update_inout(self, key, value):
        '''
        入退室の状態を更新する

        '''
        self._inout_state[key] = value

    def update_appliance(self, key, value):
        '''
        applianceの状態を更新する

        '''        
        self._applianece_state[key] = value

    def get_sensor_keys(self):
        '''
        sensorの種類を取得する

        @return dict_keys
        '''
        return self._sensor_state.keys()


    def get_inout_keys(self):
        return self._inout_state.keys()
    

    def get_appliance_keys(self):
        '''
        applianceの種類を取得する

        @return dict_keys
        '''        
        return self._applianece_state.keys()

    def get_sensor_value(self, key):
        return self._sensor_state[key]


    def get_inout_value(self, key):
        return self._inout_state[key]
    

    def get_appliance_value(self, key):
        return self._applianece_state[key]
    
    def dump(self):
        '''
        sensor, applianceの状態を辞書にして返す

        @return dict
        '''                
        return {"sensors": self._sensor_state, "inout": self._inout_state, "appliances": self._applianece_state}


    def dump_with_timestamp(self, time):
        '''
        timestampを付与して辞書で返す

        @return dict
        '''
        return {"timestamp": str(time), "sensors": self._sensor_state, "inout": self._inout_state, "appliances": self._applianece_state}


    def dump_at_now(self):
        '''
        現在の時刻をタイムスタンプとして辞書を返す

        @return dict
        '''
        tz = pytz.timezone('Asia/Tokyo')
        now = datetime.now(tz)
        return self.dump_with_timestamp(now)
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from room import parser
from room.utils.config import network_config

class InOutParserModule(parser.ParserModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder1']['back']),
            send_addr=int(network_config['forwarder2']['front']),            
            recv_title='inout',
            send_title='event',
            category='inout',
            parser=InOutParser()
        )

class InOutParser(parser.Parser):
        
    def parse(self, data):
        return [{user: 1 if data[user] == 'in' else 0 } for user in data.keys()]

    
if __name__ == "__main__":
    process = InOutParserModule()
    process.run()





#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import unittest
import json
import datetime
from room.database.mongo import MongoDB

class MongoDBTest(unittest.TestCase):
        
    @classmethod
    def setUpClass(cls):
        cls.mongo = MongoDB()

    def test_save(self):
        data = (b'', '{"appliances": {"CeilingLight1": 0.0, "CeilingLight2": 0.0, "CeilingLight3": 0.0, "airCleaner": 0.0, "airConditioner": 0.0, "airConditionerFront": 0.0, "aircleaner001": 0.0, "aircon001": 0.0, "allLight": 0.0, "allLight001": 0.0, "aroma": 0.0, "aroma001": 0.0, "dvd001": 0.0, "fan": 0.0, "fan001": 0.0, "floorLight": 0.0, "floorLight001": 0.0, "frontCeilingLight": 0.0, "hddJukeBox": 0.0, "hddRecorder": 0.0, "hotCarpet001": 0.0, "newAirCleaner": 0.0, "speaker002": 0.0, "tableLight": 0.0, "tableLight001": 0.0, "timestamp": 1349053560000, "tv": 0.0, "tv001": 0.0, "viera": 0.0, "windAirCon001": 0.0}, "sensors": {"airflow": -0.133715697, "brightness": -0.3495616132, "humidity": 0.1882105153, "out_brightness": -0.248049819, "out_humidity": 0.0966444225, "out_pressure": 0.0096008556, "out_temperature": 0.0286677364, "people": -0.1665588553, "temperature": 0.1238222059, "timestamp": 1349053560000, "w_humidity": 0.1882098846, "w_light": -0.4184434136, "w_sound": -0.0020367455, "w_temperature_c": 0.1074010302, "w_temperature_f": 0.0730542729}, "timestamp": "2015-11-16 22:47:31.285455+09:00"}')
        post = json.loads(data[1])
        self.mongo.save(post)
        
    def test_load(self):
        post = {"author": "Mike",
                "text": "My first blog post!",
                "tags": ["mongodb", "python", "pymongo"],
                "date": datetime.datetime.utcnow()}
        self.mongo.save(post)
        result = self.mongo.load()
        self.assertEqual(result[0]['author'], "Mike")
        self.assertEqual(result[0]['text'], "My first blog post!")


    @classmethod
    def tearDownClass(cls):
        cls.mongo.drop()

    
if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from room.output import OutputModule, Action
from room.utils.config import network_config

class StdOutModule(OutputModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder5']['back']),            
            recv_title='',
            action=StdOut()
        )

class StdOut(Action):

    def action(self, data):
        print(data)

if __name__ == '__main__':
    process = StdOutModule()
    process.run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import zmq
import multiprocessing
from zmq.eventloop import ioloop, zmqstream
from zmq.utils import jsonapi as json

from room.utils.log import logging

class ZmqProcess(multiprocessing.Process):
    """
    This is the base for all processes and offers utility functions 
    for setup and creating new streams.

    """
    def __init__(self):
        super().__init__()

        self.context = None
        """The ØMQ :class:`~zmq.Context` instance."""

        self.loop = None
        """PyZMQ's event loop (:class:`~zmq.eventloop.ioloop.IOLoop`)."""
        
    def setup(self):
        """
        Creates a :attr:`context` and an event :attr:`loop` for the process.

        """
        self.context = zmq.Context()
        self.loop = ioloop.IOLoop.instance()
        

    def stream(self, sock_type, addr, bind, callback=None, subscribe=b''):
        """
        Creates a :class:`~zmq.eventloop.zmqstream.ZMQStream`.

        :param sock_type: The ØMQ socket type (e.g. ``zmq.REQ``)
        :param addr: Address to bind or connect to formatted as *host:port*,
                *(host, port)* or *host* (bind to random port).
                If *bind* is ``True``, *host* may be:

                - the wild-card ``*``, meaning all available interfaces,
                - the primary IPv4 address assigned to the interface, in its
                numeric representation or
                - the interface name as defined by the operating system.

                If *bind* is ``False``, *host* may be:

                - the DNS name of the peer or
                - the IPv4 address of the peer, in its numeric representation.

                If *addr* is just a host name without a port and *bind* is
                ``True``, the socket will be bound to a random port.
        :param bind: Binds to *addr* if ``True`` or tries to connect to it
                otherwise.
        :param callback: A callback for
                :meth:`~zmq.eventloop.zmqstream.ZMQStream.on_recv`, optional
        :param subscribe: Subscription pattern for *SUB* sockets, optional,
                defaults to ``b''``.
        :returns: A tuple containg the stream and the port number.

        """
        sock = self.context.socket(sock_type)

        # addr may be `host:port` or ('host', port)
        if isinstance(addr, str):
            addr = addr.split(':')
            host, port = addr if len(addr) == 2 else (addr[0], None)

        # Bind/connect the socket
        if bind:
            if port:
                sock.bind('tcp://%s:%s' % (host, port))
            else:
                port = sock.bind_to_random_port('tcp://%s' % host)
        else:
            sock.connect('tcp://%s:%s' % (host, port))
                
        # Add a default subscription for SUB sockets
        if sock_type == zmq.SUB:
            sock.setsockopt(zmq.SUBSCRIBE, subscribe)
             
        # Create the stream and add the callback
        stream = zmqstream.ZMQStream(sock, self.loop)
        if callback:
            stream.on_recv(callback)

        return stream, int(port)
    

class  MessageHandler(object):
    """
    Base class for message handlers for a :class:`ZMQProcess`.
    
    Inheriting classes only need to implement a handler function for each message type.

    """

    def __call__(self, msg):
        """
        Gets called when a messages is recieved by the stream this handlers is
        registered at. *msg* is a list as return by
        :meth:`zmq.core.socket.Socket.recv_multipart`.
        """
        logging.info(msg)
        method = msg[1].decode('utf-8')
        data = json.loads(msg[2])
        # Get the actual message handler and call it
        if method.startswith('_'):
            raise AttributeError('%s starts with an "_"' % method)

        getattr(self, method)(data)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import unittest
import json

from room.mining.mining import MiningModule

@unittest.skip("TODO")
class MiningModule(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        'todo'
        
    @classmethod
    def tearDownClass(cls):
        'todo'

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import zmq
from abc import ABCMeta, abstractmethod

from room import zmq_base as base
from room.publisher import Publisher

class MinerModule(base.ZmqProcess):

    def __init__(self, recv_addr, send_addr, recv_title, send_title, miner):

        super().__init__()
        self.sub_stream = None        
        self.recv_addr  = recv_addr
        self.send_addr  = send_addr
        self.recv_title = recv_title
        self.send_title = send_title
        self.miner      = miner

    def setup(self):
        super().setup()
        self.sub_stream, _ = self.stream(zmq.SUB, self.recv_addr, bind=False, subscribe=self.recv_title.encode('utf-8'))
        self.sub_stream.on_recv(SubStreamHandler(self.sub_stream, self.stop, self.send_addr, self.send_title, self.miner))

    def run(self):
        self.setup()
        print('Start loop!')
        self.loop.start()

    def stop(self):
        self.loop.stop()


class SubStreamHandler(base.MessageHandler):
    
    def __init__(self, sub_stream, stop, send_addr, send_title, miner):
        super().__init__()
        self._sub_stream = sub_stream
        self._stop = stop
        self._send_title = send_title
        self._miner = miner
        self._publisher = Publisher(send_addr)

    def execute(self, data):
        result = self._miner.mining(data)
        self._publisher.send(result, self._send_title)
        
    def stop(self, data):
        self._stop()

class Miner(metaclass=ABCMeta):

    @abstractmethod
    def mining(self, data):
        raise NotImplementedError()            
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import time
from room.miner import MinerModule, Miner
from room.modules.miner.estimator.case_based_reasoning import CBR as cbr
from room.utils.config import network_config
from room.utils.config import config
from pymongo import MongoClient

class CBRModule(MinerModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder7']['back']),
            send_addr=int(network_config['forwarder5']['front']),                        
            recv_title='cbr',
            send_title='cbr',
            miner=CBR(),
        )

class CBR(Miner):

    def __init__(self):
        self.cbr = cbr()
        self.records = []
        self.db = MongoDB()
        self.load()

    def mining(self, data):
        result = self.cbr.recommend(data, self.records)
        self.add(data) # キューに保存
        self.pop() # 先頭削除
        return result

    def add(self, data):
        self.records.append(data)

    def pop(self):
        self.records.pop(0)
        
    def load(self):
        records = self.db.all()
        print(len(records), records[0])
        for record in records:
            self.add(record)
            
class MongoDB(object):

    def __init__(self):
        self._client     = MongoClient(config['mongo']['host'], int(config['mongo']['port']))
        self._db         = self._client[config['mongo']['db']]
        self._collection = self._db[config['mongo']['collection']]

    def all(self):
        return [record for record in self._collection.find()]

    def restrict(self, start_index=500000): # start_indexより後のレコードを取得する
        return [record for record in self._collection.find()[start_index:]]

    
    
if __name__ == "__main__":
    process = CBRModule()
    process.run()
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import zmq
from abc import ABCMeta, abstractmethod

from room import zmq_base as base

class OutputModule(base.ZmqProcess):

    def __init__(self, recv_addr, recv_title, action):
        super().__init__()
        self.sub_stream = None        
        self.recv_addr  = recv_addr
        self.recv_title = recv_title
        self.action     = action

    def setup(self):
        super().setup()
        self.sub_stream, _ = self.stream(zmq.SUB, self.recv_addr, bind=False, subscribe=self.recv_title.encode('utf-8'))
        self.sub_stream.on_recv(SubStreamHandler(self.sub_stream, self.stop, self.action))

    def run(self):
        self.setup()
        print('Start loop!')
        self.loop.start()

    def stop(self):
        self.loop.stop()
    
class SubStreamHandler(base.MessageHandler):

    def __init__(self, sub_stream, stop, action):
        super().__init__()
        self._sub_stream = sub_stream
        self._stop = stop
        self._action = action

    def execute(self, data):
        self._action.action(data)

    def stop(self, data):
        self._stop()


class Action(metaclass=ABCMeta):

    @abstractmethod
    def action(self, data):
        raise NotImplementedError()    
    



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

from room.buffer import BufferModule, StateHandler
from room.state import State
from room.utils.config import config
from room.utils.config import network_config

class CBRBufferModule(BufferModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder3']['back']),
            send_addr=int(network_config['forwarder7']['front']),
            recv_title='buffer',
            send_title='cbr',
            state_handler=CBRStateHandler(),
            period=int(config['cbr_buffer']['interval_ms'])
        )
        
class CBRStateHandler(StateHandler):

    def __init__(self):
        self._records = []
    
        
    def dump(self):

        sum = {}
        sum['appliances'] = {}
        sum['sensors'] = {}
        sum['inout'] = {}
        
        result = {}
        result['appliances'] = {}
        result['sensors'] = {}
        result['inout'] = {}
        
        num = len(self._records)
        mid = num // 2
        
        for record in self._records:
            for key, value in record['appliances'].items():
                if key not in sum['appliances']: sum['appliances'][key] = value
                else: sum['appliances'][key] += value
            for key, value in record['sensors'].items():
                if key not in sum['sensors']: sum['sensors'][key] = value                
                else: sum['sensors'][key] += value            
            for key, value in record['inout'].items():
                if key not in sum['inout']: sum['inout'][key] = value                                
                else: sum['inout'][key] += value            
            
        for key in sum['appliances'].keys():
            result['appliances'][key] = sum['appliances'][key] / num
        for key in sum['sensors'].keys():
            result['sensors'][key] = sum['sensors'][key] / num
        for key in sum['inout'].keys():
            result['inout'][key] = sum['inout'][key] / num            

        print(str(sum))
        result['timestamp'] = self._records[mid]['timestamp']
                
        self._records.clear()
        
        return result

    def buffering(self, data):
        self._records.append(data)

if __name__ == "__main__":
    process = CBRBufferModule()
    process.run()
    



#!/usr/bin/env python3
# -*- coding: utf-8- 

import zmq
import time

class Subscriber():

    def __init__(self, bind_addr, subscriber=b''):
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.SUB)
        self.socket.connect('tcp://' + bind_addr)
        self.socket.setsockopt(zmq.SUBSCRIBE, subscriber)
        time.sleep(1.0)
        
    def recv(self):
        return self.socket.recv_multipart()

    def stop(self):
        self.socket.close()
        self.context.term()
     




from setuptools import setup, find_packages


install_requires = [
    'chainer',
    'numpy',
    'pymongo',
    'pyzmq',
    'tornado',
    'pypandoc',
]


CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries",
    "Topic :: Utilities",
    "License :: OSI Approved :: MIT License",
]

packages = [
    'room',
    'room.buffer',
    'room.database',
    'room.filter',
    'room.forwarder',
    'room.miner',
    'room.output',
    'room.parser',
    'room.publisher',
    'room.zmq_base',
]


# pypandocを使ってREADME.mdをrstに変換する。最初からrstで書いた場合は不要。
try:  
    import pypandoc
    read_md = lambda f: pypandoc.convert(f, 'rst')
except ImportError:  
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

dist = setup(  
        name='room',                        # パッケージ名
        version='0.1.3',                    # バージョン
        author='takatorino',                # 作者の名前
        author_email='tktr3w5e0@gmail.com', # 連絡先
        url='https://github.com/takatori',  # GitHubのリポジトリとか
        license='MIT',                      # ライセンス
        description="A framework of stream data maining for smart home",
        keywords='data mining, smart home, api',
        classifiers=CLASSIFIERS, # 詳細は http://pypi.python.org/pypi?:action=list_classifiers を参照
        packages=find_packages(exclude=['examples', 'tests']),
        include_package_data=True,
        zip_safe=False,
        install_requires=install_requires,     # インストールする依存パッケージ
)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import zmq
import time
from zmq.utils import jsonapi as json

from room.utils import log

class Publisher(object):
    '''
    ZeroMQによるPub/SubパターンのPublisherを実装するためのクラス
    
    '''
    
    def __init__(self, port, title=''):
        '''
        ソケットの作成
        @param port: 送り先ポート
        @param title: メッセージの先頭に付与される文字列、subscriberはこれを参照してパケットを取得するか破棄するかを決める
        '''
        assert isinstance(port, int), '[type error]: {0} != int'.format(port)
        assert isinstance(title, str), '[type error]: {0} != str'.format(title)
        
        self.title = title
        context = zmq.Context() # Context作成
        self._sock = context.socket(zmq.PUB) # Publish用のソケット作成
        log.logging.info(port)
        self._sock.connect("tcp://%s:%s" % ("localhost", port))
        time.sleep(1.0) # SUB は定期的に PUB に接続を見に行くので、少し待つ必要が有る

    def send(self, data, title='', method='execute'):
        '''
        メッセージをsubscriberに送信する

        @param title: メッセージの先頭に付与される文字列、subscriberはこれを参照してパケットを取得するか破棄するかを決める
        @param method: subscriberで呼び出されるメソッド名
        @param data: メッセージ本体
        '''
        assert isinstance(title, str), '[type error]: {0} != str'.format(title)
        assert isinstance(method, str), '[type error]: {0} != str'.format(method)
        assert isinstance(data, str) or isinstance(data, dict) or isinstance(data, list), '[type error]: {0} != str || dict || list'.format(data)

        if title:
            self.title = title  # send時にtitleが設定されていればその値を使用する
            
        payload = json.dumps(data, sort_keys=True) # obj を JSON 形式の str に直列化
        self._sock.send_multipart([self.title.encode('utf-8'), method.encode('utf-8'), payload])

    def stop(self):
        '''
        ソケットを閉じる

        '''
        self._sock.close()



# -*- coding: utf-8 -*-

import uuid
import urllib.parse
from tornado import gen
from tornado.httpclient import AsyncHTTPClient, HTTPRequest

from room.output import OutputModule, Action
from room.utils.config import config, network_config

class HttpOutputModule(OutputModule):

    def __init__(self):
        super().__init__(
            recv_addr='localhost:{0}'.format(network_config['forwarder5']['back']),            
            recv_title='',
            action=HttpOut()
        )

        
class HttpOut(Action):
    @gen.coroutine
    def action(self, data):
        http_client = AsyncHTTPClient()
        for recommend in data:
            appliance = recommend['appliance']
            method    = recommend['method']
            headers = {'Content-Type': 'application/x-www-form-urlencoded'}            
            payload = {"recommend_id": str(uuid.uuid4()), "appliance": appliance, "method": method}
            urlencoded_payload = urllib.parse.urlencode(payload)
            request = HTTPRequest(config['http_output']['url'], 'POST', headers, body=urlencoded_payload)
            yield http_client.fetch(request)

if __name__ == '__main__':
    process = HttpOutputModule()
    process.run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import zmq
import time
import os
import signal
from  multiprocessing import Process

from room.utils import log
from room.utils.config import network_config

def forward(front_port, backend_port):

    context = zmq.Context(1)
    # Socket facing clients
    frontend = context.socket(zmq.SUB)
    frontend.bind("tcp://*:%s" % front_port)

    frontend.setsockopt(zmq.SUBSCRIBE, b"")

    # Socket facing services
    backend = context.socket(zmq.PUB)
    backend.bind("tcp://*:%s" % backend_port)
    
    try:
        log.logging.info("Start forwarding from %s to %s", front_port, backend_port)
        zmq.device(zmq.FORWARDER, frontend, backend)

    except Exception as e:
        log.logging.error(e)
        print("[bringing down zmq device]")

    finally:
        frontend.close()
        backend.close()
        context.term()

        
def main():
    processes = []
    
    try:
        for section in network_config.sections():
            front_port = int(network_config[section]['front'])
            back_port  = int(network_config[section]['back'])
            p = Process(target=forward, args=(front_port, back_port,))
            p.start()
            processes.append(p)

        for p in processes:
            p.join()

    except Exception as e:
        log.logging.info(e)

    finally:
        for p in processes:
            if p.is_alive():
                os.kill(p.pid, signal.SIGINT)
                p.join()
                
if __name__ == '__main__':
    main()
